/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package net.jazz.rtcext.workitem.extensions.service;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.internal.common.ITeamBuildRequestService;
import com.ibm.team.build.internal.common.ITeamBuildService;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IItemsResponse;
import com.ibm.team.process.common.advice.IProcessReport;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.runtime.IOperationParticipant;
import com.ibm.team.process.common.advice.runtime.IParticipantInfoCollector;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.service.AbstractService;
import com.ibm.team.workitem.common.ISaveParameter;
import com.ibm.team.workitem.common.model.IState;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.Identifier;

/**
 * 
 * If on the save of a work item of a certain type, the state has changed to a
 * certain state, run the specified build.
 * 
 * It is critical to understand that operation participants are managed as
 * singletons by the process component. Therefore, their methods, most notably
 * the run method must be reentrant. Operation participants must not rely on any
 * instance state variables (i.e. non-static fields).
 * 
 * While rare, it is occasionally the case that the complexity of the operation
 * to be performed and the number and interactions of methods and their data
 * interdependencies will present a case where the use of instance state
 * variables is highly desirable. In this case, another class will need to be
 * defined and an instance of that class created for each invocation of the run
 * method. The run method can then delegate the operation to the instance of
 * this second class. This second class can use instance state variables for its
 * implementation.
 * 
 */
public class BuildOnStateChangeParticipant extends AbstractService implements
		IOperationParticipant {

	/**
	 * If a process uses the build on state change participant, this run method
	 * is called by the Jazz process engine whenever a work item is saved.
	 * 
	 * @param operation
	 *            the operation in which to participate
	 *            <p>
	 *            The operation provides the runtime context for the
	 *            participant. This participant will use it to get information
	 *            about the work item that is being saved.
	 *            </p>
	 * @param participantConfig
	 *            the configuration element which configures this participant;
	 *            this corresponds to the XML element which declares this
	 *            participant in the process specification/customization.
	 *            <p>
	 *            At this point, this participant does not require any process
	 *            specification, so this parameter is not used. This will change
	 *            in the future.
	 *            </p>
	 * @param collector
	 *            the collector into which any info should be added
	 *            <p>
	 *            Participants report back failures and success via this
	 *            collector.
	 *            </p>
	 * @param monitor
	 *            the monitor used to report progress and receive cancellation
	 *            requests
	 *            <p>
	 *            Generally speaking, server side participants, like this one,
	 *            do not need to report progress to the monitor. Client side
	 *            participants should report progress. Checking for cancellation
	 *            after long operations performed by the participant is a good
	 *            practice on either client or server.
	 *            </p>
	 * @throws TeamRepositoryException
	 *             if one of the invoked service api throws it
	 */
	public void run(AdvisableOperation operation,
			IProcessConfigurationElement participantConfig,
			IParticipantInfoCollector collector, IProgressMonitor monitor)
			throws TeamRepositoryException {
		/*
		 * There are several reasons not to request a build. This code is
		 * structured to perform the most common and fastest cases first. The
		 * idea is to find out as quickly as possible that you have nothing to
		 * do. Depending on how the participant is configured (which process
		 * area(s), iteration(s) or roles it affects), it could be called for
		 * all work item saves that occur in the using project area.
		 */

		/*
		 * First check that the operation data is work item save data.
		 */
		Object data = operation.getOperationData();
		ISaveParameter saveParameter = null;
		if (data instanceof ISaveParameter) {
			saveParameter = (ISaveParameter) data;
			/*
			 * If the state id has not changed, do not build.
			 */
			IWorkItem newState = (IWorkItem) saveParameter.getNewState();
			Identifier<IState> newStateId = newState.getState2();
			Identifier<IState> oldStateId = null;
			IWorkItem oldState = (IWorkItem) saveParameter.getOldState();
			if (oldState != null) // New work item check.
				oldStateId = oldState.getState2();
			if ((newStateId != null) && !(newStateId.equals(oldStateId))) {
				/*
				 * If the work item is not of the proper type, do not build.
				 * Note that the work item type and state are hard coded, as is
				 * the build definition id. We will change that later. That is
				 * when we will start using the participantConfig parameter.
				 */
				String newType = newState.getWorkItemType();
				if (newType.equals("com.ibm.team.apt.workItemType.story")) {
					/*
					 * Finally, if the new state is the target state, build.
					 */
					if (newState.getState2().getStringIdentifier().equals(
							"com.ibm.team.apt.story.tested")) {
						build("our.integration.build", collector);
					}
				}
			}
		}
	}

	/**
	 * Request the specified build.
	 * 
	 * @param buildId
	 *            the build definition id to run
	 * @param collector
	 *            the collector into which any info should be added
	 *            <p>
	 *            This participant will report back any failures it discovers,
	 *            for example, if the build definition does not exist.
	 *            </p>
	 * @throws TeamRepositoryException
	 *             if one of the invoked service api throws it
	 */
	protected void build(String buildId, IParticipantInfoCollector collector)
			throws TeamRepositoryException {
		/*
		 * The team build service is used to access team build information
		 * (engines, definitions, results, etc). We use it to get the desired
		 * build definition.
		 */
		ITeamBuildService buildService = getService(ITeamBuildService.class);
		IBuildDefinition buildDef = buildService.getBuildDefinition(buildId);

		/*
		 * If the build definition was found, try to run the build.
		 */
		if (buildDef != null) {
			/*
			 * The team build request service is used to create, access and
			 * manipulate build requests. We will request a new build. The true
			 * and false in the build request parameters indicate that it is
			 * okay if multiple builds of this type are in the queue and that
			 * this is not a personal build. The first null indicates that we
			 * have no build properties to add or override (we will change this
			 * later). The second null means there are no build properties to
			 * delete.
			 */
			ITeamBuildRequestService brService = getService(ITeamBuildRequestService.class);
			IItemsResponse response = brService.requestBuild(buildDef, null,
					null, true, false);

			/*
			 * If the build request has been submitted, report success back. It
			 * will show up in the team advisor if success reports are not being
			 * filtered out and the show details tree is expanded.
			 */
			if ((response != null) && (response.getFirstClientItem() != null)) {
				String description = NLS
						.bind(
								"A new build request for build definition ''{0}'' was submitted.",
								buildId);
				IReportInfo info = collector.createInfo(
						"Build request successful", description);
				collector.addInfo(info);
			} else {
				/*
				 * The build request could not be created, report this back as
				 * an error. An error report will stop the work item save from
				 * succeeding and will show up in the team advisor.
				 */
				String description = NLS
						.bind(
								"The build request for build definition ''{0}'' could not be honored. It is likely that a build engine is not associated with the definition.",
								buildId);
				IReportInfo info = collector.createInfo(
						"Unable to request build", description);
				info.setSeverity(IProcessReport.ERROR);
				collector.addInfo(info);
			}
		} else {
			/*
			 * The build definition was not found, report this back as an error.
			 * An error report will stop the work item save from succeeding and
			 * will show up in the team advisor.
			 */
			String description = NLS
					.bind(
							"The build request for build definition ''{0}'' could not be found.",
							buildId);
			IReportInfo info = collector.createInfo("Unable to request build",
					description);
			info.setSeverity(IProcessReport.ERROR);
			collector.addInfo(info);
		}
	}

}
